/* File: gethostnames.c
 * 
 * Hatem Nassrat
 * B00393388 - Dalhousie University
 *
 * Part of: A Fault tolerance Implementation for an OLAP system
 *
 * sub-application to create a file on each of the nodes /tmp/pcc05 dir
 * named hostfile. i.e. /tmp/pcc05/hostfile that contains the hostnames
 * of live nodes
 */
#include <mpi.h>
#include <stdio.h>

#define IC_CREATE_TAG 5

int main(int argc, char * argv[])
{
    int i, my_rank, p, f;
    int size,qcount;
    char hostname[BUFSIZ];
    char buf[BUFSIZ];
    MPI_Comm *my_comm;
    MPI_Status status;
    struct timeval start,stop;
    FILE* hostfile;

    /*MPI INIT*/
    
    MPI_Init(&argc,&argv);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &p);
    f=p;


        /* create intercommunicators and set error handlers */
    if(my_rank==0) //manager process
    {
        my_comm = (MPI_Comm*) malloc ((p-1)*sizeof(MPI_Comm));
        for(i=1;i<p;i++)
        {
            MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, i, IC_CREATE_TAG, &my_comm[i-1]);
            MPI_Comm_set_errhandler(my_comm[i-1], MPI_ERRORS_RETURN);
        }
    }
    else
    {
        my_comm = (MPI_Comm*)malloc(sizeof(MPI_Comm));
        MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, 0, IC_CREATE_TAG, my_comm);
        MPI_Comm_set_errhandler(*my_comm, MPI_ERRORS_RETURN);
    }
    /* Each p has a link to manager and vice versa */

    /*END OF: MPI INIT*/
    
    if(my_rank != 0)
    {
        
        for(i=2;i<argc;i++)
            if(my_rank==atoi(argv[i])) exit(0);
            else f--;

        gethostname(hostname,BUFSIZ-1);
        //send my name to node 0 (root node)
        MPI_Send(hostname,BUFSIZ,MPI_CHAR, 0, 1, *my_comm);
        MPI_Recv(hostname,BUFSIZ,MPI_CHAR, 0, 1, *my_comm, &status);        
        sprintf(buf, "scp %s:/tmp/pcc05/hostfile /tmp/pcc05/hostfile",hostname);
        system(buf);
    }
    else{
    
        for(i=2;i<argc;i++)
        {
            //MPI_Comm_free(&my_comm[atoi(argv[i])-1]);
            my_comm[atoi(argv[i])-1]=NULL;
            f--;
        }

        gethostname(hostname,BUFSIZ-1);
        hostfile = fopen("/tmp/pcc05/hostfile","w");
        fprintf(hostfile,"%s\n",hostname);
        for(i=1;i<p;i++)
        {
            if(!my_comm[i-1])
                fprintf(hostfile,"\n");
            else
            {
                MPI_Recv(hostname,BUFSIZ,MPI_CHAR,0,1,my_comm[i-1],&status);   
                fprintf(hostfile,"%s\n",hostname);
            }
        }
        
        gethostname(hostname,BUFSIZ-1);
        for(i=1;i<p;i++)
            MPI_Send(hostname,BUFSIZ,MPI_CHAR, 0, 1, my_comm[i-1]);
    }
    
    MPI_Finalize();
    exit(0);
}

